package admin

import (
	"code.gitea.io/gitea/models"
	"code.gitea.io/gitea/modules/base"
	"code.gitea.io/gitea/modules/context"
	"code.gitea.io/gitea/modules/log"
	"code.gitea.io/gitea/routers/response"
	"code.gitea.io/gitea/services/cloudbrain/resource"
	"net/http"
)

const (
	tplResourceQueue         base.TplName = "admin/resources/queue"
	tplResourceSpecification base.TplName = "admin/resources/specification"
	tplResourceScene         base.TplName = "admin/resources/scene"
)

func GetQueuePage(ctx *context.Context) {
	ctx.Data["PageIsAdmin"] = true
	ctx.Data["PageIsAdminResources"] = true
	ctx.Data["PageIsAdminResourcesQueue"] = true
	ctx.HTML(200, tplResourceQueue)
}

func GetSpecificationPage(ctx *context.Context) {
	ctx.Data["PageIsAdmin"] = true
	ctx.Data["PageIsAdminResources"] = true
	ctx.Data["PageIsAdminResourcesSpecification"] = true
	ctx.HTML(200, tplResourceSpecification)
}

func GetScenePage(ctx *context.Context) {
	ctx.Data["PageIsAdmin"] = true
	ctx.Data["PageIsAdminResources"] = true
	ctx.Data["PageIsAdminResourcesScene"] = true
	ctx.HTML(200, tplResourceScene)
}

func GetResourceQueueList(ctx *context.Context) {
	page := ctx.QueryInt("page")
	cluster := ctx.Query("cluster")
	aiCenterCode := ctx.Query("center")
	computeResource := ctx.Query("resource")
	accCardType := ctx.Query("card")
	list, err := resource.GetResourceQueueList(models.SearchResourceQueueOptions{
		ListOptions:     models.ListOptions{Page: page, PageSize: 20},
		Cluster:         cluster,
		AiCenterCode:    aiCenterCode,
		ComputeResource: computeResource,
		AccCardType:     accCardType,
	})
	if err != nil {
		log.Error("GetResourceQueueList error.%v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.SuccessWithData(list))
}

func AddResourceQueue(ctx *context.Context, req models.ResourceQueueReq) {
	req.IsAutomaticSync = false
	req.CreatorId = ctx.User.ID
	err := resource.AddResourceQueue(req)
	if err != nil {
		log.Error("AddResourceQueue error. %v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.Success())
}

func UpdateResourceQueue(ctx *context.Context, req models.ResourceQueueReq) {
	queueId := ctx.ParamsInt64(":id")
	//only CardsTotalNum permitted to change
	err := resource.UpdateResourceQueue(queueId, req.CardsTotalNum)
	if err != nil {
		log.Error("UpdateResourceQueue error. %v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.Success())
}

func GetResourceSpecificationList(ctx *context.Context) {
	page := ctx.QueryInt("page")
	queue := ctx.QueryInt64("queue")
	status := ctx.QueryInt("status")
	list, err := resource.GetResourceSpecificationList(models.SearchResourceSpecificationOptions{
		ListOptions: models.ListOptions{Page: page, PageSize: 20},
		QueueId:     queue,
		Status:      status,
	})
	if err != nil {
		log.Error("GetResourceSpecificationList error.%v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.SuccessWithData(list))
}

func AddResourceSpecification(ctx *context.Context, req models.ResourceSpecificationReq) {
	req.IsAutomaticSync = false
	req.CreatorId = ctx.User.ID
	err := resource.AddResourceSpecification(req)
	if err != nil {
		log.Error("AddResourceQueue error. %v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.Success())
}

func UpdateResourceSpecification(ctx *context.Context, req models.ResourceSpecificationReq) {
	id := ctx.ParamsInt64(":id")
	action := ctx.Query("action")

	var err error
	switch action {
	case "edit":
		if req.UnitPrice < 0 {
			ctx.JSON(http.StatusOK, response.ServerError("param error"))
			return
		}
		//only UnitPrice permitted to change
		err = resource.UpdateUnitPrice(id, req.UnitPrice)
	}

	if err != nil {
		log.Error("UpdateResourceSpecification error. %v", err)
		ctx.JSON(http.StatusOK, response.ServerError(err.Error()))
		return
	}
	ctx.JSON(http.StatusOK, response.Success())
}
